/*
**  File	: db.h
**
** -- Header File --
*/

#ifndef __DB_H__
#define __DB_H__

#ifdef DB_USE_SCHED
#include "hmsched.h"
#endif /* DB_USE_SCHED */

#define DB_MAX_DATA_PATH		256
#define DB_MAX_LOCK_PATH		512

#define DB_SORT_ASCENDING		1
#define DB_SORT_DESCENDING		2

#define DB_SORT_CASE_SENSITIVE		1
#define DB_SORT_NON_CASE_SENSITIVE	2

#define DB_SEARCH_CASE_SENSITIVE	1
#define DB_SEARCH_NON_CASE_SENSITIVE	2

/* Open Paramaters */
#define DB_NONE				0x0000
#define DB_LOCK				0x0001
#define DB_BLOCK			0x0002
#define DB_SAVE_LOCK			0x0004

typedef struct DataBaseParam
{
	long			p_len;
	void			*p;
	struct DataBaseParam	*next;
	struct DataBaseParam	*prev;
} dbParam;

typedef struct DataBaseInfo
{
	long			num_params;
	struct DataBaseParam	*p_first;
	struct DataBaseParam	*p_cur;
	struct DataBaseParam	*p_last;
	struct DataBaseInfo	*next;
	struct DataBaseInfo	*prev;
	char			first_access;
} dbInfo;

typedef struct DataBaseName
{
	char	name[80];
	long	version;
	long	records;
} dbName;

typedef struct	DataBase
{
	unsigned long		time_created;
	char			table_name[80];
	dbName			*dbname;
	dbInfo			*dbcur;
	dbInfo			*dbinfo;
	dbInfo			*dbinfo_end;
	dbInfo			*search_cur;
	char			first_access;
	char			*search_text;
	char			search_start;
	char			search_case;
	int			search_param;
	long			flags;
	char			lock_file[DB_MAX_LOCK_PATH];
} dBase;

typedef struct DataBaseList
{
	char	*name;
	dBase	*db;

	struct DataBaseList	*next;
	struct DataBaseList	*prev;
} dBaseList_t;

#ifdef DB_USE_SCHED
typedef struct dBTransState
{
	int		fd;

	dBase		*db;
	dbInfo		*dbi;

	u_long		last;

	SchedFD_t	fd_save;

	void		(*callback)(void *);
	void		*arg;
} dBTransState_t;
#endif /* DB_USE_SCHED */

int		db_set_data_path(char *dpath);
void		db_set_verbose(int x);
void		db_set_byte_swap(int x);
void		db_set_default_null_string(char *s);
dBase *		db_find_dbase(char *dbname);
dBase *		db_create(char *dbname, char *desc, long version);
int		db_save(dBase *db);
#ifdef DB_USE_SCHED
int		db_send(int fd, dBase *db, void (*callback)(void *), void *arg);
int		db_recv(int fd, void (*callback)(void *), void *arg);
#else /* not DB_USE_SCHED */
int		db_send(int fd, dBase *db);
int		db_recv(int fd);
#endif /* not DB_USE_SCHED */
int		db_send_dbi(int fd, dbInfo *dbi);
int		db_recv_dbi(int fd, dBase *db);
dBase *		db_open(char *dbname, long flags);
void		db_close(dBase *db);
void		db_close_all(void);
char *		db_name(dBase *db);
long		db_version(dBase *db);
long		db_records(dBase *db);
dbInfo *	db_new(dBase *db);
int		db_remove(dBase *db, dbInfo *dbi);
void		db_start(dBase *db);
void		db_end(dBase *db);
dbInfo *	db_cur(dBase *db);
dbInfo *	db_next(dBase *db);
dbInfo *	db_prev(dBase *db);
dbInfo *	db_recall(dBase *db, int n);
dbInfo *	db_find(dBase *db, int n, char *s, int c);
long		db_num_params(dbInfo *dbi);
dbParam *	db_param_new(dbInfo *dbi, long p_len, void *p);
int		db_param_update(dbParam *dbp, long p_len, void *p);
int		db_param_remove(dbInfo *dbi, dbParam *dbp);
void		db_param_start(dbInfo *dbi);
void		db_param_end(dbInfo *dbi);
dbParam *	db_param_cur(dbInfo *dbi);
dbParam *	db_param_next(dbInfo *dbi);
dbParam *	db_param_prev(dbInfo *dbi);
void		db_dump(dbInfo *dbi, int (*print_func)(char *fmt, ...));
void		db_dump_param(dBase *db, int param);
char *		db_param_string(dbParam *dbp);
void *		db_param_value(dbParam *dbp);
long		db_param_length(dbParam *dbp);
dbParam *	db_param_recall(dbInfo *dbi, int n);
dbParam *	db_param_find(dbInfo *dbi, char *s);
char *		db_get_low(char *s);
int 		db_sort_by_param(dBase *db, int param, int param2, int param3, int dir, int case_sensitive);
int		db_sort_params(dbInfo *dbi, int dir, int case_sensitive);
dBase *		db_copy(dBase *db, char *new_name);
dbInfo *	db_copy_dbi(dbInfo *dbi, dBase *db);
int		db_search_set(dBase *db, char c, int n, char *s);
int		db_search_matches_left(dBase *db);
dbInfo *	db_search_next(dBase *db);
long		db_byte_swap(long l);
#ifdef DB_USE_SCHED
void		db_register_cmds(int (*add_cmd)(char *, char *, char *,
					int (*handler)(int, char **), u_long),
						int (*print_func)(char *fmt, ...)) ;
void		db_fd_event(int fd, void *arg, int r, int w, int e);
#endif /* DB_USE_SCHED */


extern dBaseList_t	*_db_list;
extern dBaseList_t	*_db_list_end;

#endif /* __DB_H__ */
